@@ -38,30 +38,29 @@ RELINKING_WARNING = 'Relinking will always happen on source changes.'
 
 def dummy_syms(outfilename: str) -> None:
     """Just touch it so relinking happens always."""
-    with open(outfilename, 'w'):
+    with open(outfilename, 'w', encoding='utf-8'):
         pass
 
 def write_if_changed(text: str, outfilename: str) -> None:
     try:
-        with open(outfilename, 'r') as f:
+        with open(outfilename, encoding='utf-8') as f:
             oldtext = f.read()
         if text == oldtext:
             return
     except FileNotFoundError:
         pass
-    with open(outfilename, 'w') as f:
+    with open(outfilename, 'w', encoding='utf-8') as f:
         f.write(text)
 
 def print_tool_warning(tools: T.List[str], msg: str, stderr: T.Optional[str] = None) -> None:
-    global TOOL_WARNING_FILE
     if os.path.exists(TOOL_WARNING_FILE):
         return
-    m = '{!r} {}. {}'.format(tools, msg, RELINKING_WARNING)
+    m = f'{tools!r} {msg}. {RELINKING_WARNING}'
     if stderr:
         m += '\n' + stderr
     mlog.warning(m)
     # Write it out so we don't warn again
-    with open(TOOL_WARNING_FILE, 'w'):
+    with open(TOOL_WARNING_FILE, 'w', encoding='utf-8'):
         pass
 
 def get_tool(name: str) -> T.List[str]:
@@ -104,7 +103,7 @@ def gnu_syms(libfilename: str, outfilename: str) -> None:
         dummy_syms(outfilename)
         return
     result = [x for x in output.split('\n') if 'SONAME' in x]
-    assert(len(result) <= 1)
+    assert len(result) <= 1
     # Get a list of all symbols exported
     output = call_tool('nm', ['--dynamic', '--extern-only', '--defined-only',
                               '--format=posix', libfilename])
@@ -119,7 +118,7 @@ def gnu_syms(libfilename: str, outfilename: str) -> None:
         # Store the size of symbols pointing to data objects so we relink
         # when those change, which is needed because of copy relocations
         # https://github.com/mesonbuild/meson/pull/7132#issuecomment-628353702
-        if line_split[1].upper() in ('B', 'G', 'D') and len(line_split) >= 4:
+        if line_split[1].upper() in {'B', 'G', 'D'} and len(line_split) >= 4:
             entry += [line_split[3]]
         result += [' '.join(entry)]
     write_if_changed('\n'.join(result) + '\n', outfilename)
@@ -161,7 +160,7 @@ def openbsd_syms(libfilename: str, outfilename: str) -> None:
         dummy_syms(outfilename)
         return
     result = [x for x in output.split('\n') if 'SONAME' in x]
-    assert(len(result) <= 1)
+    assert len(result) <= 1
     # Get a list of all symbols exported
     output = call_tool('nm', ['-D', '-P', '-g', libfilename])
     if not output:
@@ -171,6 +170,24 @@ def openbsd_syms(libfilename: str, outfilename: str) -> None:
     result += [' '.join(x.split()[0:2]) for x in output.split('\n') if x and not x.endswith('U ')]
     write_if_changed('\n'.join(result) + '\n', outfilename)
 
+def freebsd_syms(libfilename: str, outfilename: str) -> None:
+    # Get the name of the library
+    output = call_tool('readelf', ['-d', libfilename])
+    if not output:
+        dummy_syms(outfilename)
+        return
+    result = [x for x in output.split('\n') if 'SONAME' in x]
+    assert len(result) <= 1
+    # Get a list of all symbols exported
+    output = call_tool('nm', ['--dynamic', '--extern-only', '--defined-only',
+                              '--format=posix', libfilename])
+    if not output:
+        dummy_syms(outfilename)
+        return
+
+    result += [' '.join(x.split()[0:2]) for x in output.split('\n')]
+    write_if_changed('\n'.join(result) + '\n', outfilename)
+
 def cygwin_syms(impfilename: str, outfilename: str) -> None:
     # Get the name of the library
     output = call_tool('dlltool', ['-I', impfilename])
@@ -268,6 +285,10 @@ def gen_symbols(libfilename: str, impfilename: str, outfilename: str, cross_host
         osx_syms(libfilename, outfilename)
     elif mesonlib.is_openbsd():
         openbsd_syms(libfilename, outfilename)
+    elif mesonlib.is_freebsd():
+        freebsd_syms(libfilename, outfilename)
+    elif mesonlib.is_netbsd():
+        freebsd_syms(libfilename, outfilename)
     elif mesonlib.is_windows():
         if os.path.isfile(impfilename):
             windows_syms(impfilename, outfilename)
@@ -289,12 +310,12 @@ def gen_symbols(libfilename: str, impfilename: str, outfilename: str, cross_host
             mlog.warning('Symbol extracting has not been implemented for this '
                          'platform. ' + RELINKING_WARNING)
             # Write it out so we don't warn again
-            with open(TOOL_WARNING_FILE, 'w'):
+            with open(TOOL_WARNING_FILE, 'w', encoding='utf-8'):
                 pass
         dummy_syms(outfilename)
 
 def run(args: T.List[str]) -> int:
-    global TOOL_WARNING_FILE
+    global TOOL_WARNING_FILE  # pylint: disable=global-statement
     options = parser.parse_args(args)
     if len(options.args) != 4:
         print('symbolextractor.py <shared library file> <import library> <output file>')
